# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
import logging
import os
import StringIO
import ConfigParser

LOG = logging.getLogger( "Audit" )

AUDIT_CONFIG_FILENAME = "navigator.client.properties"
AUDIT_SERVER_URL_KEY = "navigator.server.url"
AUDIT_ROLE_TYPE_KEY = "navigator.client.roleType"
AUDIT_LOG_DIR_KEY = "audit_event_log_dir"
SERVICE_NAME_KEY = "navigator.client.serviceName"
AUDIT_SERVER_URL_KEY = "navigator.server.url"
AUDIT_BATCH_TIMEOUT = "navigator.batch.timeout"
DEFAULT_BATCH_TIMEOUT = 5
AUDIT_MIN_BATCH_SIZE = "navigator.batch.min_size"
DEFAULT_MIN_BATCH_SIZE = 10

# This is the section we use to convert properties to ini file format
AUDIT_CONFIG_HEADER = "[audit]"  
AUDIT_CONFIG_SECTION = "audit"

def make_audit_plugin(process_name):
  """
  Creates an audit plugin that will collect audit events and send them to
  the audit server.
  """
  class EmptyPlugin(object):
    def refresh(self, conf_dir):
      pass
    def stop_audit_monitoring(self):
      pass
    
  if process_name == 'impala-IMPALAD':
    from cmf.audit.impalad import ImpaladAuditPlugin
    return ImpaladAuditPlugin()
  else:
    return EmptyPlugin()
    
def parse_audit_configuration(conf_dir, audit_component):
  """
  Parses a audit configuration file and returns a ConfigParser object
  with parsed key and values. If the file is not found (means auditing 
  server isn't present), or if the configuration file is empty (means auditing
  isn't enabled), None is returned.
  """
  conf_file = os.path.join(conf_dir, AUDIT_CONFIG_FILENAME)
  if os.path.isfile(conf_file) and os.path.getsize(conf_file) > 0:
    try:
      # convert properties file to add a dummy section so that we can use
      # ConfigParser to parse it
      conf_ini_string = AUDIT_CONFIG_HEADER + '\n' + open(conf_file, 'r').read()
      conf_fd = StringIO.StringIO(conf_ini_string)
      config = ConfigParser.SafeConfigParser()
      config.readfp(conf_fd)
      conf = AuditConfig(config)
      LOG.info("Read auditing config for role %s. Audit server url: %s" %
              (conf.role_type(), conf.server_url()))
      if conf.server_url():
        return conf
      else:
        return None
    except IOError, ex:
      LOG.exception("Error when accessing %s file." % (conf_file, ))
      return None
  else:
    LOG.debug("Auditing is not enabled for %s as audit config file is "
              "missing or empty." % (audit_component))
    return None

class AuditConfig(object):
  """
  Instance of this class store all the audit configuration for easy access.
  """
  def __init__(self, conf):
    self._conf = conf

  def log_dir(self):
    return self._conf.get(AUDIT_CONFIG_SECTION, AUDIT_LOG_DIR_KEY)
  
  def server_url(self):
    try:
      return self._conf.get(AUDIT_CONFIG_SECTION, AUDIT_SERVER_URL_KEY)
    except ConfigParser.NoOptionError:
      return None
    
  def role_type(self):
    try:
      return self._conf.get(AUDIT_CONFIG_SECTION, AUDIT_ROLE_TYPE_KEY)
    except ConfigParser.NoOptionError:
      return None
  
  def service_name(self):
    return self._conf.get(AUDIT_CONFIG_SECTION, SERVICE_NAME_KEY)
  
  def batch_min_size(self):
    try:
      min_batch_size = self._conf.get(AUDIT_CONFIG_SECTION, AUDIT_MIN_BATCH_SIZE)
      return int(min_batch_size)
    except ConfigParser.NoOptionError:
      LOG.debug("Returning default batch size of %d" % (DEFAULT_MIN_BATCH_SIZE,))
      return DEFAULT_MIN_BATCH_SIZE
    
  def batch_timeout(self):
    """
    Timeout in seconds to wait for batch_min_size audit events to arrive. After
    this time send whatever events have been received.
    """
    try:
      timeout = self._conf.get(AUDIT_CONFIG_SECTION, AUDIT_BATCH_TIMEOUT)
      return int(timeout)/1000
    except  ConfigParser.NoOptionError:
      LOG.debug("Returning default batch timeout of %d" % (DEFAULT_BATCH_TIMEOUT,))
      return DEFAULT_BATCH_TIMEOUT